/*
	Module:		Map Module
	Programmer:	Michael Scott Stuart
	Version:	2.2.7
	Modified:	10/22/07
	Created:	10/08/07
	Library:	CRM32Pro 4.9.9.0

	2.2.7:	
	
	[10/21/07][MSS][ALL]:	Changed Xoffset and YOffset from float to int.
	[10/18/07][MSS][ALL]:	Updated header comments.

	10/3/07 - COMPLETELY rewrote the Map Manager. The Map Manager is now just a namespace
			which contains the Map Class. The Map Class handles everything on it's own,
			which helped me drop ALOT of now useless code. Along with this: Layer support 
			has finally (yay!) been added. I may even drop the namespace, but I do love
			how it organizes the Map Class in the Object Browser. :)
	4/02/07 - Added get_map2_layers for the MAP2 format.
	3/21/07 - Added MAP2 format which supports layers. MAP2 Tile data format has been expanded to long, which leaves
			some room (2 bytes) for a future implementation of tile flags. The MAP2 format is handled by a MAP2 
			manager. (So technically, there are two map managers for you to use.. and init/close.. :) ) MAP2 Format
			as of this update is untested. (Man I am really tired and lazy, sorry. It shows so much in the code..)
			- MAP2 format manager functions are named *_map2 or *_map2_*.
	3/12/07 - Removed all multiple-map support (which wasn't really added in the first place, it just "looked" like it).
			The map manager only handles one map at a time.
	2/27/07 - Fixed some awful code in render_map that told it to try and render all maps in the map array.
	2/06/07 - Recalc'd height to add an extra row of tiles in render_map(). 
	1/27/07 - Added b to filemode when saving and loading maps. This seems to have fixed the bug where a map
			would not load properly and either most of it, or all of it would be blank. (I hope it did).
	1/26/07	- added "if(!maps) return;" to render_map.
			- Fixed Get and Set, They were checking x > x_size & y > y_size instead of x >= x_size & y >= y_size.
			Should fix some possibly dangerous situations, and maybe some of the errors Rob noticed.
			- Resize was bugged the hell out. ;) it is now fixed and working.
	1/25/07 - render_map now draws black rectangles (tiles) for out-of-bounds tiles. (tiles not in the map).
	1/24/07 - Removed CRM32Pro library link code. it's redundant.  
			- Added render_map to render a Map using a specified CRM32Pro_CTile.
			- Changed MAX_MAPS 8 to MAX_MAPS 32. Why not? :) 
	1/23/07 - All function names use either of these naming conformances: *_map or *_map_*.
	1/20/07 - Initially coded. 
*/

#pragma once

#include "Crm32Pro.h"

#define MAP_NAME_SIZE			32 // The length of the map name. (Eases saving/loading)
#define MAP_MAX_LAYERS			10 // The maximum amount of Tile Layers allowed.
#define MAP_MAX_WIDTH			1024 // The maximum width (columns) of Tiledata.
#define MAP_MAX_HEIGHT			1024 // The maximum height (rows) of Tiledata. 

#define DEFAULT_MAP_WIDTH		32 // This is the default map width if one isn't provided.
#define DEFAULT_MAP_HEIGHT		32 // This is the default map height if one isn't provided.
#define DEFAULT_MAP_LAYERS		2 // This is the default map layer count if one isn't provided.
#define DEFAULT_MAP_NAME		"Map" // This is the default map name is one isn't provided.

#define ERR_OK					0 // No error has occured.
#define ERR_SETTILE_X_RANGE		-1 // The SetTile X parameter is out of range.
#define ERR_SETTILE_Y_RANGE		-2 // The SetTile Y parameter is out of range.
#define ERR_SETTILE_LAYER_RANGE -3	// The SetTile Layer parameter is out of range.

#define ERR_GETTILE_X_RANGE		-4 // The GetTile X parameter is out of range.
#define ERR_GETTILE_Y_RANGE		-5 // The GetTile Y parameter is out of range.
#define ERR_GETTILE_LAYER_RANGE -6	// The GetTile Layer parameter is out of range.

#define ERR_RESET_WIDTH			-7 // The Reset Width is less than 1.
#define ERR_RESET_HEIGHT		-8 // The Reset Height is less than 1.
#define ERR_RESET_LAYERS		-9 // The Reset Layers is less than	1.
#define ERR_RESET_ALLOCATION	-10 // Reset failed to allocate memory for Tiledata.

#define ERR_RESIZE_ALLOCATION	-11 // Resize failed to allocate memory for Tiledata.
#define ERR_RESIZE_WIDTH		-12 // The Resize Width is less than 1.
#define ERR_RESIZE_HEIGHT		-13 // The Resize Height is less than 1.
#define ERR_RESIZE_LAYERS		-14 // The Resize Layers is less than 1.
#define ERR_RESIZE_RESET		-15 // Resize called Reset but Reset failed.

#define ERR_SAVE_FILENAME		-16 // Save failed to verify the filename or file.
#define ERR_SAVE_FILE			-17 // Save::file or Save::file->_ptr is invalid.
#define ERR_SAVE_FAILED			-18 // Save failed to correctly write to the file.
									// (the hard drive is probably out of free space.)

#define ERR_LOAD_FILENAME		-25 // Load failed to verify the filename or file.
#define ERR_LOAD_FILE			-26 // Load::file or Load::file->_ptr is invalid.
#define ERR_LOAD_FORMAT			-27	// Loaded map file format is incorrect. 

#define ERR_RENDER_TILESET		-30 // Render determined the Tileset was invalid.




// This	is the Map object. Map objects encapsulate map data and implement functionality
// for said map data.
class Map
{

public:
	// Default & Parametric	constructor.
	Map(int	Width=32, int Height=32, int Layers=2);
	// The destructor.
	~Map(void);
	// Loads a properly	formatted map file
	long Load(const char* mapFilename);
	// Saves a Map's data to a properly	formatted data file.
	long Save(const char* mapFilename);
	// Resizes a map and retains as	much previous map data as possible.	(Irreversible tile removal can occur!)
	long Resize(int	Width, int Height, int Layers);
	// Resets a	map's tile data. All previous Tile data	is lost.
	long Reset(void);
	// Gets	a specific tile	on a specific Layer	from the Map.
	long GetTile(int X,	int	Y, int Layer);
	// Sets	a specific tile	on a specific Layer	in the Map.	
	long SetTile(int X,	int	Y, int Layer, long Tile);
	// Using the map data and a	CRM32Pro Tileset the map is	rendered.
	// RWidth is Render	Width, and RHeight is Render Height. These two parameters determine	how
	// much	of the map will	be drawn. If Layer is 0	all	layers will	be rendered. Otherwise Layer
	// will	be the only	layer to use for rendering.	
	long Render(CRM32Pro_CTile*	Tileset, int RWidth, int RHeight, int Layer);
	// Returns the Error received formatted as a string message
	char* ErrorToStr(int Err);
private:
	// The width of	the	map
	int	Width;
	// The Height of the map
	int	Height;
	// The amount of Layers	for	the	map.
	int	Layers;
	// The name	of the map
	char Name[MAP_NAME_SIZE];
	// The Tile	data for every layer of	the	map.
	long* Tiledata;	
public:
	// A simple validity check. ALWAYS Check this property after the constructor has
	// been called. Initialized will tell you if all-is-well and that you can continue 
	// using your map object.
	bool Initialized;
	// The X Offset	of the map.	Used for "scrolling" the map.
	int Xoffset;
	// The Y Offset	of the map.	Used for "scrolling" the map
	int Yoffset;
	// Returns the Map Width
	int GetWidth(void) { return this->Width; }
	// Returns the Map Height
	int GetHeight(void) { return this->Height; }
	// Returns the Map Height
	int GetLayers(void) { return this->Layers; }
	// Returns the name of the Map.
	char* GetName(void) { return this->Name; }
};