//
// Module: MAP MANAGER
// Author: Michael Scott Stuart
// Notice: Copyright 2007 Michael Scott Stuart
// Status: Advanced

//
// IMPLEMENTATION
//

//
// HEADERS/DEFINITIONS/LIBS IMPLEMENTATION 
//
#include "map_man.h"

// <string> & <vector> class header include
#include <string>
#include <vector>
using namespace std;

//
// DATA IMPLEMENTATION
//
typedef long MAP_LONG_TILE;
typedef short MAP_SHORT_TILE;

// map structure.
typedef struct smap_s
{
	smap_s(void)	
	{		
		x_size = 0;
		y_size = 0;
		fx_off = 0.0f;
		fy_off = 0.0f;
		data = NULL;
	}

	bool Assigned(void)
	{		
		if(data) return true;	
		return false;
	}
	
	bool New(const int xsize, const int ysize)
	{				
		x_size = xsize;
		y_size = ysize;
		if(x_size<1 || y_size<1) return false;
		if(Assigned()) Delete();
		data = new short[ (x_size * y_size) ];	
		for(int x=0; x < x_size; ++x)
			for(int y=0; y < y_size; ++y)
				Set(-1, x, y);
		if(!data) return false;
		return true;
	}		

	bool Load(char* mapfilename)
	{
		if(!mapfilename) return false;	
		if(Assigned()) Delete();
		FILE* file;
		file = fopen(mapfilename, "rb");
		if(file==NULL) return false;
		fread(&x_size, sizeof(int), 1, file);
		fread(&y_size, sizeof(int), 1, file);
		fclose(file);
		// can't alloc mem while file is open.. for some reason new will fail.
		New(x_size, y_size);
		if(!Assigned())
			return false;
		file = fopen(mapfilename, "rb");
		if(file==NULL) return false;
		fread(&x_size, sizeof(int), 1, file);
		fread(&y_size, sizeof(int), 1, file);
		fread(&fx_off, sizeof(float), 1, file);
		fread(&fy_off, sizeof(float), 1, file);
		fread(data, sizeof(short) * (x_size * y_size), 1, file);
		fclose(file);

		#ifdef _DEBUG
			ILogSystem.Msg(LOG_NORMAL, "   smap_t::Load(%s) - Tiles Loaded:\n", mapfilename);
			for(int x=0; x < x_size; ++x)
				for(int y=0; y < y_size; ++y)
					ILogSystem.Msg(LOG_NORMAL, "%i, ", Get(x, y));
			ILogSystem.Msg(LOG_NORMAL, "\n");
		#endif // _DEBUG

		return true;
	}

	bool Save(char* mapfilename)
	{
		if(!mapfilename) return false;
		if(!Assigned()) return false;
		FILE* file;
		file = fopen(mapfilename, "wb");
		if(file==NULL) return false;
		fwrite(&x_size, sizeof(int), 1, file);
		fwrite(&y_size, sizeof(int), 1, file);
		fwrite(&fx_off, sizeof(float), 1, file);
		fwrite(&fy_off, sizeof(float), 1, file);
		fwrite(data, sizeof(short) * (x_size * y_size), 1, file);
		fclose(file);

		#ifdef _DEBUG
			ILogSystem.Msg(LOG_NORMAL, "   smap_t::Load(%s) - Tiles Loaded:\n", mapfilename);
			for(int x=0; x < x_size; ++x)
				for(int y=0; y < y_size; ++y)
					printf("%i, ", Get(x, y));
			ILogSystem.Msg(LOG_NORMAL, "\n");
		#endif // _DEBUG

		return true;
	}

	bool Resize(const int xsize, const int ysize) 
	{		
		if(x_size<1 || y_size<1) return false;
		if(xsize<1 || ysize<1) return false;
		int ox_size = x_size;								// store the old x_size for tdata
		int oy_size = y_size;								// store the old y_size for tdata
		short* tdata = new short[ox_size * oy_size];
		if(!tdata) return false;
		for(int x=0; x<ox_size; ++x)
			for(int y=0; y<oy_size; ++y)
				tdata[ (ox_size * y) + x ] = Get(x, y);				
		Delete();											// resets ALL data (x_size, y_size = 0)..												
		New(xsize, ysize);
		if(!Assigned()) return false;
		int max_x = (ox_size > x_size) ? x_size : ox_size;
		int max_y = (oy_size > y_size) ? y_size : oy_size;
		for(int x=0; x < max_x; ++x)
			for(int y=0; y < max_y; ++y)
				Set( tdata[ ((ox_size * y) + x) ], x, y);				
		return true;
	}

	void Delete(void)
	{
		x_size = 0;
		y_size = 0;		
		fx_off = 0.0f;
		fy_off = 0.0f;
		delete [] data;
		data = NULL;
	}

	int Get(int x, int y)
	{
		if(!Assigned()) return -1;
		if(x<0 || x>=x_size) return -1;
		if(y<0 || y>=y_size) return -1;
		return data[ ((y * x_size) + x) ];
	}

	int Set(int tile, int x, int y)
	{		
		if(!Assigned()) return -1;
		if(x<0 || x>=x_size) return -1;
		if(y<0 || y>=y_size) return -1;
		data[ ((y * x_size) + x) ] = tile;
		return data[ ((y * x_size) + x) ];
	}	

	int x_size;
	int y_size;
	float fx_off;
	float fy_off;
	short* data;
} smap_t;

// map2 structure.
/*
	This map structure handles Layers. Every layer is of the same x & y size. Tile data has been expanded to a long
	in order to make room for tile flags.
*/
//typedef struct smap2_tile_s
//{
//	short tile;
//	long flags;
//} smap2_tile_t;

typedef struct smap2_s
{
	smap2_s(void)	
	{		
		layers = 0;
		x_size = 0;
		y_size = 0;
		fx_off = 0.0f;
		fy_off = 0.0f;
		data = NULL;
	}

	bool Assigned(void)
	{		
		if(data) return true;	
		return false;
	}
	
	bool New(const int xsize, const int ysize, const int lyrs)
	{
		layers = lyrs;
		x_size = xsize;
		y_size = ysize;
		if(x_size<1 || y_size<1 || layers < 1) return false;
		if(Assigned()) Delete();
		data = new MAP_LONG_TILE[ layers * (x_size * y_size) ];
		for(int layer=1; layer <= layers; ++layer)
			for(int x=0; x < x_size; ++x)
				for(int y=0; y < y_size; ++y)
					Set(-1, x, y, layer);
		if(!data) return false;
		return true;
	}

	bool Load(char* mapfilename)
	{
		if(!mapfilename) return false;	
		if(Assigned()) Delete();
		FILE* file;
		file = fopen(mapfilename, "rb");
		if(file==NULL) return false;
		fread(&layers, sizeof(int), 1, file);
		fread(&x_size, sizeof(int), 1, file);
		fread(&y_size, sizeof(int), 1, file);
		fclose(file);
		// can't alloc mem while file is open.. for some reason new will fail.
		New(x_size, y_size, layers);
		if(!Assigned())
			return false;
		file = fopen(mapfilename, "rb");
		if(file==NULL) return false;
		fread(&layers, sizeof(int), 1, file);
		fread(&x_size, sizeof(int), 1, file);
		fread(&y_size, sizeof(int), 1, file);
		fread(&fx_off, sizeof(float), 1, file);
		fread(&fy_off, sizeof(float), 1, file);
		fread(data, sizeof(MAP_LONG_TILE) * (layers * (x_size * y_size)), 1, file);
		fclose(file);

		#ifdef _DEBUG
			ILogSystem.Msg(LOG_NORMAL, "   smap_t::Load(%s) - Tiles Loaded:\n", mapfilename);

			for(int layer=1; layer <= layers; ++layer)
			{
				ILogSystem.Msg(LOG_NORMAL, "   layer %i:\n", layer);
				for(int x=0; x < x_size; ++x)
					for(int y=0; y < y_size; ++y)
						ILogSystem.Msg(LOG_NORMAL, "%i, ", Get(layer, x, y));
			}

			ILogSystem.Msg(LOG_NORMAL, "\n");
		#endif // _DEBUG

		return true;
	}

	bool Save(char* mapfilename)
	{
		if(!mapfilename) return false;
		if(!Assigned()) return false;
		FILE* file;
		file = fopen(mapfilename, "wb");
		if(file==NULL) return false;
		fwrite(&layers, sizeof(int), 1, file);
		fwrite(&x_size, sizeof(int), 1, file);
		fwrite(&y_size, sizeof(int), 1, file);
		fwrite(&fx_off, sizeof(float), 1, file);
		fwrite(&fy_off, sizeof(float), 1, file);
		fwrite(data, sizeof(MAP_LONG_TILE) * (layers * (x_size * y_size)), 1, file);
		fclose(file);

		#ifdef _DEBUG
			ILogSystem.Msg(LOG_NORMAL, "   smap_t::Load(%s) - Tiles Loaded:\n", mapfilename);
			for(int layer=1; layer <= layers; ++layer)
			{
				ILogSystem.Msg(LOG_NORMAL, "   layer %i:\n", layer);
				for(int x=0; x < x_size; ++x)
					for(int y=0; y < y_size; ++y)
						printf("%i, ", Get(x, y, layer));
			}
			ILogSystem.Msg(LOG_NORMAL, "\n");
		#endif // _DEBUG

		return true;
	}

	bool Resize(const int xsize, const int ysize, const int lyrs) 
	{		
		/*
			TODO: IMPLEMENT RESIZE FOR MAP2 LAYERED DATA STRUCTURE
		*/
		if(x_size<1 || y_size<1) return false;
		if(xsize<1 || ysize<1) return false;
		if(lyrs<1) return false;
		int ox_size = x_size;								// store the old x_size for tdata
		int oy_size = y_size;								// store the old y_size for tdata
		int o_layers = layers;								// store the old layers for tdata
		MAP_LONG_TILE* tdata = new MAP_LONG_TILE[ layers * (ox_size * oy_size) ];
		if(!tdata) return false;
		for(int layer=1; layer <= o_layers; ++layer)
			for(int x=0; x<ox_size; ++x)
				for(int y=0; y<oy_size; ++y)
					tdata[ layer * ((ox_size * y) + x) ] = Get(x, y, layer);
		Delete();											// resets ALL data (x_size, y_size = 0)..												
		New(xsize, ysize, lyrs);
		if(!Assigned()) return false;
		int max_x = (ox_size > x_size) ? x_size : ox_size;
		int max_y = (oy_size > y_size) ? y_size : oy_size;
		int max_layer = (o_layers > layers) ? layers : o_layers;
		for(int layer=1; layer<=max_layer; ++layer)
			for(int x=0; x < max_x; ++x)
				for(int y=0; y < max_y; ++y)
					Set( tdata[ layer * (((ox_size * y) + x)) ], x, y, layer);
		return true;
	}

	void Delete(void)
	{
		layers = 0;
		x_size = 0;
		y_size = 0;
		fx_off = 0.0f;
		fy_off = 0.0f;
		delete [] data;
		data = NULL;
	}

	MAP_LONG_TILE Get(int x, int y, int layer)
	{
		if(!Assigned()) return -1;
		if(layer < 1 || layer > layers) return -1;
		if(x<0 || x>=x_size) return -1;
		if(y<0 || y>=y_size) return -1;
		return data[ layer * ((y * x_size) + x) ];
	}

	MAP_LONG_TILE Set(int tile, int x, int y, int layer)
	{		
		if(!Assigned()) return -1;
		if(layer < 1 || layer > layers) return -1;
		if(x<0 || x>=x_size) return -1;
		if(y<0 || y>=y_size) return -1;
		data[ layer * ((y * x_size) + x) ] = tile;
		return data[ layer * ((y * x_size) + x) ];
	}	

	int layers;
	int x_size;
	int y_size;
	float fx_off;
	float fy_off;	
	MAP_LONG_TILE* data;
} smap2_t;

smap2_t* mm_layered_map = NULL;

//
// FUNCTIONS IMPLEMENTATION
//

// Initializes the map manager.
bool init_map_manager(void)
{	
	if(mm_layered_map)
		close_map_manager();
	
	mm_layered_map = new smap2_t();
	return true;
}

// Closes the map manager. Frees memory used by the Maps.
void close_map_manager(void)
{
	if(mm_layered_map->Assigned())	
		mm_layered_map->Delete();

	delete mm_layered_map;
	mm_layered_map = NULL;
}

// Loads a Map from file.
bool load_map(char* mapfilename)
{	
	if(mm_layered_map)	
		return mm_layered_map->Load(mapfilename);
	return false;
}

// Saves a Map to file.
bool save_map(char* mapfilename)
{
	if(mm_layered_map)	
		return mm_layered_map->Save(mapfilename);	
	return false;
}

// Creates a new empty Map.
bool new_map(int mapxsize, int mapysize, int layers)
{
	if(mm_layered_map)
	{
		return mm_layered_map->New(mapxsize, mapysize, layers);
	}
	return false;
}

// Frees the map.
bool free_map(void)
{
	if(mm_layered_map)
	{
		if(mm_layered_map->Assigned())
		{
			mm_layered_map->Delete();
			return true;
		}
	}
	return false;
}

// Resizes the Map.
bool resize_map(int newxsize, int newysize, int newlayers)
{
	if(mm_layered_map)
	{
		if(mm_layered_map->Assigned())
			return mm_layered_map->Resize(newxsize, newysize, newlayers);
		else
			return mm_layered_map->New(newxsize, newysize, newlayers);
	}
	return false;
}

// Sets the Map's offsets.
void set_map_offsets(float fx_off, float fy_off)
{
	if(mm_layered_map)
	{
		if(mm_layered_map->Assigned())
		{
			mm_layered_map->fx_off = fx_off;
			mm_layered_map->fy_off = fy_off;
		}
	}
}

// Returns the Map's offsets.
void get_map_offsets(float& fx_off, float& fy_off)
{	
	if(mm_layered_map)
	{
		if(mm_layered_map->Assigned())
		{
			fx_off = mm_layered_map->fx_off;
			fy_off = mm_layered_map->fy_off;
		}
	}
}

// Returns the Map's X and Y size.
void get_map_size(int& xsize, int& ysize)
{
	if(mm_layered_map)
	{
		if(mm_layered_map->Assigned())
		{
			xsize = mm_layered_map->x_size;
			ysize = mm_layered_map->y_size;
		}
	}
}

// Returns the Map2's number of layers.
int get_map_layers(void)
{
	if(mm_layered_map)
	{
		if(mm_layered_map->Assigned())		
			return mm_layered_map->layers;		
	}
	return -1;
}

// Sets the Map Tile at Map Position X, Y.  Returns False if Failed.
bool set_map_xy(int x, int y, int tile, int layer)
{
	if(mm_layered_map)
	{
		if(mm_layered_map->Assigned())
		{
			if( x<0 || y<0) return false;
			if( layer < 1 || layer > mm_layered_map->layers) return false;
			if( x > mm_layered_map->x_size || y > mm_layered_map->y_size) return false;
			mm_layered_map->Set(tile, x, y, layer);
			return true;
		}
	}	
	return false;
}

// Returns a Map Tile at Map Position X, Y. Returns False if Failed. 
bool get_map_xy(int& tile, int x, int y, int layer)
{
	if(mm_layered_map)
	{
		if(mm_layered_map->Assigned())
		{
			if( x<0 || y<0) return false;
			if( layer < 1 || layer > mm_layered_map->layers) return false;
			if( x > mm_layered_map->x_size || y > mm_layered_map->y_size) return false;
			tile = mm_layered_map->Get(x, y, layer);
			return true;
		}
	}
	return false;
}

// Renders the Map to CRM32Pro.screen using pTileset for the tile graphics.
// WhichLayer = 0 = Draw all layers.
void render_map(CRM32Pro_CTile* pTileset, int WhichLayer) 
{
	if(!mm_layered_map) return;
	if(!mm_layered_map->Assigned()) return;
	if(!pTileset) return;
	if(!pTileset->GetSurface()) return;

	// here's the fun part. (I really should comment this.)
	int ts_x = 0;
	int ts_y = 0;
	float fx_off = mm_layered_map->fx_off;
	float fy_off = mm_layered_map->fy_off;
	pTileset->GetOffset(&ts_x, &ts_y);
	int width = CRM32Pro.screen->w / ts_x + 1; 
	int height = CRM32Pro.screen->h / ts_y + 2;

	/*
		Render ALL layers.
	*/
	if(WhichLayer==0)
	{
		for(int layer=1; layer <= mm_layered_map->layers; ++layer)		
			for(int x=0; x < width; ++x)
				for(int y=0; y < height; ++y)
				{
					int tx = 0;
					int ty = 0;
					tx = x + ((int)fx_off / ts_x);
					ty = y + ((int)fy_off / ts_y);
					int tile = 0;
					if( tx > -1 && tx < mm_layered_map->x_size && 
						ty > -1 && ty < mm_layered_map->y_size )
						tile = mm_layered_map->Get(tx, ty, layer);
					else
					{
#ifdef _DEBUG
						SDL_Rect rect;
						rect.x = x * ts_x - ((int)fx_off % ts_x);
						rect.y = y * ts_y - ((int)fy_off % ts_y);
						rect.w = ts_x;
						rect.h = ts_y;
						SDL_FillRect(CRM32Pro.screen, &rect, 0x010101);
						continue;
#endif // _DEBUG_
					}
					if(tile>-1)
					{
						int xpos = x * ts_x - ((int)fx_off % ts_x);
						int ypos = y * ts_y - ((int)fy_off % ts_y);
						pTileset->SetPosition(xpos, ypos);

						/*
						if the tile doesn't exist in the tileset, we need to prevent
						this particular tile from being drawn with this tileset.
						*/
						if(!pTileset->Draw(CRM32Pro.screen, tile)) // -1 If failed.
						{
							// TODO: remember this tile # to avoid drawing it again.
						}

					}
				}
	}

	/*
	Render only one specific layer.
	*/
	else
	{
		for(int x=0; x < width; ++x)
			for(int y=0; y < height; ++y)
			{
				int tx = 0;
				int ty = 0;
				tx = x + ((int)fx_off / ts_x);
				ty = y + ((int)fy_off / ts_y);
				int tile = 0;
				if( tx > -1 && tx < mm_layered_map->x_size && 
					ty > -1 && ty < mm_layered_map->y_size )
					tile = mm_layered_map->Get(tx, ty, WhichLayer);
				else
				{
#ifdef _DEBUG
					SDL_Rect rect;
					rect.x = x * ts_x - ((int)fx_off % ts_x);
					rect.y = y * ts_y - ((int)fy_off % ts_y);
					rect.w = ts_x;
					rect.h = ts_y;
					SDL_FillRect(CRM32Pro.screen, &rect, 0x010101);
					continue;
#endif // _DEBUG_
				}
				if(tile>-1)
				{
					int xpos = x * ts_x - ((int)fx_off % ts_x);
					int ypos = y * ts_y - ((int)fy_off % ts_y);
					pTileset->SetPosition(xpos, ypos);

					/*
					if the tile doesn't exist in the tileset, we need to prevent
					this particular tile from being drawn with this tileset.
					*/
					if(!pTileset->Draw(CRM32Pro.screen, tile)) // -1 If failed.
					{
						// TODO: remember this tile # to avoid drawing it again.
					}

				}
			}
	}
}

// Module: MAP MANAGER