#include "resources.h"

// Tileset Scroll Up button handle.
int Tileset_Scroll_Up;
// Tileset Scrool Down button handle.
int Tileset_Scroll_Down;
// DPF filename edit box
int DPF_Edit_Box;
// Map filename edit box
int Map_Edit_Box;
// DPF Set Button
int DPF_Set;
// Tileset Pick Menu Button
int Tileset_Menu;
// Layer Count Edit Box
int Layer_Edit_Box;
// Prev Layer Button
int Prev_Layer;
// Next Layer Button
int Next_Layer;

// Map Load/Save/Size/Set Buttons
int Map_Load;
int Map_Save;
int Map_XSize;
int Map_YSize;
int MapSize_Set;

// Editor Font. Highlighted White colored Font. (Bright yellow)
CRM32Pro_CFont* Font_White_High;
// Editor Font. White colored Font.
CRM32Pro_CFont* Font_White;

// Tileset Picking Menu Images.
// Top Menu Image
SDL_Surface* Top_Menu;
// Bottom Menu Image
SDL_Surface* Bottom_Menu;
// Tileset Pick Item Image
SDL_Surface* Tspick_Item;
// Tileset Pick Item High Image
SDL_Surface* Tspick_Item_High;

// Loads and initializes all editor resources. (gfx/buttons, etc..)
void loadResources(void)
{
	// Load Editor resources from it's DPF and initialize them.

	// Fonts..
	Font_White_High = new CRM32Pro_CFont();								// Fonts - Global Useage
	Font_White = new CRM32Pro_CFont();
	Font_White_High->Load(EDITOR_DPF, "FontWhiteHigh");
	Font_White->Load(EDITOR_DPF, "FontWhite");	

	// Menu Buttons/Images..
	Top_Menu = IImage->Load(EDITOR_DPF, "Top_Menu");					// Top Menu Image 
	Bottom_Menu = IImage->Load(EDITOR_DPF, "Bottom_Menu");				// Bottom Menu Image
	Tspick_Item = IImage->Load(EDITOR_DPF, "TSPick_Item");				// Tileset Picking Item Image.
	Tspick_Item_High = IImage->Load(EDITOR_DPF, "TSPick_Item_High");	// Tileset Picking Highlighted Item Image.
	
	// DPF Buttons..
	DPF_Edit_Box = IButton->Load(EDITOR_DPF, "Edit_128");				// DPF Filename Edit Box.	
	DPF_Set = IButton->Load(EDITOR_DPF, "Set_Button");					// Set DPF Button	

	// Tileset Menu Button..
	Tileset_Menu = IButton->Load(EDITOR_DPF, "Tileset_Pick");			// Pick Tileset Button

	// Tileset Palette Buttons..
	Tileset_Scroll_Up = IButton->Load(EDITOR_DPF, "TilesetScrollUp");	// Buttons - Tileset Palette
	Tileset_Scroll_Down = IButton->Load(EDITOR_DPF, "TilesetScrollDown");	
		
	// Map Buttons..
	Map_Edit_Box = IButton->Load(EDITOR_DPF, "Edit_128");				// Map filename edit box.
	Map_Load = IButton->Load(EDITOR_DPF, "Load_Button");				// Map Load/Save Buttons
	Map_Save = IButton->Load(EDITOR_DPF, "Save_Button");					
	Map_XSize = IButton->Load(EDITOR_DPF, "Edit_32");					// Map X Size edit box
	Map_YSize = IButton->Load(EDITOR_DPF, "Edit_32");					// Map Y size edit box
	MapSize_Set = IButton->Load(EDITOR_DPF, "Set_Button");				// Map resize button
	
	// Layer Buttons..
	Layer_Edit_Box = IButton->Load(EDITOR_DPF, "Edit2_32");				// Layer_Edit_Box
	Prev_Layer = IButton->Load(EDITOR_DPF, "Prev_Layer");				// Previous Layer button
	Next_Layer = IButton->Load(EDITOR_DPF, "Next_Layer");				// Next Layer button	

	// Set the font for every text box..
	IButton->SetTextFont(DPF_Edit_Box, Font_White);	
	IButton->SetTextFont(Map_Edit_Box, Font_White);	
	IButton->SetTextFont(Layer_Edit_Box, Font_White);
	IButton->SetTextFont(Map_XSize, Font_White);
	IButton->SetTextFont(Map_YSize, Font_White);

	// Set the  text rectangles for every text box...
	SDL_Rect name_rect;
	name_rect.x = 2;
	name_rect.y = 4;
	name_rect.w = 110;
	name_rect.h = 10;
	SDL_Rect size_rect;
	size_rect.x = 2;
	size_rect.y = 4;
	size_rect.w = 42;
	size_rect.h = 10;

	IButton->SetTextRect(Map_Edit_Box, &name_rect); 
	IButton->SetTextRect(DPF_Edit_Box, &name_rect); 
	IButton->SetTextRect(Map_XSize, &size_rect);
	IButton->SetTextRect(Map_YSize, &size_rect);
	IButton->SetTextRect(Layer_Edit_Box, &size_rect);

	// Set the positions for every button/image..
	IButton->SetPosition(Tileset_Scroll_Up, 0, 0);	
	IButton->SetPosition(Tileset_Scroll_Down, 0, CRM32Pro.screen->h - 16);	

	// TOP MENU Postions..
	IButton->SetPosition(DPF_Set,		TOPMENU_DPF_SET_XPOS,	0);
	IButton->SetPosition(DPF_Edit_Box,	TOPMENU_DPF_EDIT_XPOS,	0);
	IButton->SetPosition(Tileset_Menu,	TOPMENU_TSPICK_XPOS,	0);

	// BOTTOM MENU Positions..
	IButton->SetPosition(Map_Load,		BOTMENU_MAP_LOAD_XPOS,		CRM32Pro.screen->h - 32);
	IButton->SetPosition(Map_Save,		BOTMENU_MAP_SAVE_XPOS,		CRM32Pro.screen->h - 32);
	IButton->SetPosition(Map_Edit_Box,	BOTMENU_MAP_EDIT_XPOS,		CRM32Pro.screen->h - 32);
	IButton->SetPosition(Map_XSize,		BOTMENU_MAP_XSIZE_XPOS,		CRM32Pro.screen->h - 32);
	IButton->SetPosition(Map_YSize,		BOTMENU_MAP_YSIZE_XPOS,		CRM32Pro.screen->h - 32);
	IButton->SetPosition(MapSize_Set,	BOTMENU_MAPSIZE_SET_XPOS,	CRM32Pro.screen->h - 32);	
	IButton->SetPosition(Prev_Layer,	BOTMENU_LAYER_PREV_XPOS,	CRM32Pro.screen->h - 16);
	IButton->SetPosition(Next_Layer,	BOTMENU_LAYER_NEXT_XPOS,	CRM32Pro.screen->h - 16);
	IButton->SetPosition(Layer_Edit_Box,BOTMENU_LAYER_COUNT_XPOS,	CRM32Pro.screen->h - 16);
}
