//
// Module: SIMPLE TILESET MANAGER
// Author: Michael Scott Stuart
// Notice: Copyright 2007 Michael Scott Stuart
// Status: DEBUG
/*
	2/02/07 - Fixed two more memory leaks in load_tileset, which were easier to fix than I thought. :)
	2/01/07 - Fixed memory leak in load_tileset by adding IStuffDPF->FreeBlock() after a IStuffDPF->LoadBLock().
	1/24/07 - added is_loaded_tileset() to check validity of Tilesets.
			- changed load_tileset(char*) to load_tileset(CRM32Pro_CTile, char*). You can now send a 
			Tileset that has been initialized and/or loaded and load_tileset will handle it accordingly. 
			This should help avoid memory leaks.
			- Some debugging done, everything seems to work great after debugging.			
	1/23/07 - Initial coding completed.

			// From	CRM32Pro forums	at stratos-ad.com (from	TheAzazel, the CRM32Pro	Author):
			//
			// Hi again	Mike, 
			//
			// I have never	thought	someone	will need to know the internal structure of	the	CTile resources	
			// but Im happy	to know	I was wrong	 
			//
			// The tiles are stored	with typeblock = "TILE2". 
			//
			// The struct is: 
			// X surface size: 4 bytes 
			// Y surface size: 4 bytes 
			// X position: 4 bytes 
			// Y position: 4 bytes 
			// Tileset flag: 4 bytes 
			// Tileset X offset: 4 bytes 
			// SDL Pixel format	(SDL_PixelFormat): 40 bytes	
			// Raw pixels data:	DPFBlockdata.size -	128. 
			// Tileset Y offset: 4 bytes (this was added in	the	new	tile v2) 
			// Reserved	data: 60 bytes to future improvements. 
			//
			// Well, hope this can help	you, any other you need, please, let me	know! 
			//
			// Thanks to The Azazel	for	the	help. Now if he'd just give	me the data	structure for sprites
			// I'd be set. ;)
*/

#ifndef _STSMANH_
#define _STSMANH_

//
// INTERFACE HEADERS
//
// CRM32Pro header include
#ifndef CRM32Pro_H
#include "Crm32Pro.h"
#endif // CRM32Pro_H

//
// INTERFACE DEFINITIONS
//
#define STS_FILENAME_SIZE	128
#define STS_MAX_TILESETS	128

//
// INTERFACE FUNCTION PROTOTYPES
//

// Sets the STS DPF. Builds a Internal Tileset name list to help with accessing Tilesets.
bool set_sts_dpf(char* dpffilename);

// Returns the total number of Tilesets found with set_sts_dpf().
int get_sts_total(void);

// Returns the name of a Tileset. Returns NULL if failed.
char* get_sts_name(int sts);

// Returns true if the tileset is a useable tileset (if it has a valid SDL_Surface). False if otherwise.
bool is_loaded_tileset(CRM32Pro_CTile* ptile);

// Loads (and creates) a Tileset and returns it's pointer. Returns NULL if Failed.
// You must delete the Tileset yourself when you are finished with it.
CRM32Pro_CTile* load_tileset(CRM32Pro_CTile* ptile, char* dpfname);

#endif // _STSMANH_

// Module: CRM32Pro SIMPLE TILESET MANAGER